/**
 * XXX: This file should be compiled into 32-bit object file, then use objcopy
 * to convert the object file into 64-bit object file, finally it can be linked
 * with other 64-bit object files.
 *
 * FIXME: Rewrite this file in asm and put source code into boot.s
 */

namespace kernel {
	
typedef unsigned long long PageTableItem;

void fillPageTable(PageTableItem pageTable[], PageTableItem pageDirectory[],
		PageTableItem pagePointer[], PageTableItem pageMap[]) {
	PageTableItem fields = (1 << 0) /* P  Page presents */
						| (1 << 1) /* R/W Read & write access */
						| (1 << 2) /* U/S User & system page */
						;
	int i = 0;
	for (i = 0; i < 1024 * 1024; ++i) {
		pageTable[i] = (i << 12) | fields;
	}

	for (i = 0; i < 2048; ++i) {
		pageDirectory[i] = (((long)pageTable & 0xfffff000) + i * 4096)
							| fields;
	}

	for (i = 0; i < 4; ++i) {
		pagePointer[i] = (((long)pageDirectory & 0xfffff000) + i * 4096)
							| fields;
	}

	pageMap[0] = ((long)pagePointer & 0xfffff000) | fields;
}

void fillSystemSegmentDescriptor(unsigned long limit, unsigned long address,
		unsigned short descriptor[], unsigned long type) {
	descriptor[0] = (unsigned short)limit;
	descriptor[1] = (unsigned short)address;
	descriptor[2] = (unsigned short)((unsigned short)type
										| (0x00FF & (address >> 16)));
	descriptor[3] = (unsigned short)(((address >> 16) & 0xFF00)
										| ((limit >> 16) & 0x0F));
	descriptor[4] = 0; descriptor[5] = 0;
	descriptor[6] = 0; descriptor[7] = 0;
}

struct InterruptDescriptor {
	unsigned short offset15_0;
	unsigned short selector;
	unsigned short type;
	unsigned short offset31_16;
	unsigned long offset63_32;
	unsigned long zero;

	InterruptDescriptor() : offset63_32(0), zero(0) {}
} __attribute__((packed));

void fillInterruptDescriptorTable(struct InterruptDescriptor idt[],
		unsigned long address[], long selector) {
	for (int i = 0; i < 32; ++i) {
		idt[i].offset15_0 = (unsigned short)address[i];
		idt[i].selector = selector;
		idt[i].type = 0x8e00;
		idt[i].offset31_16 = (unsigned short)(address[i] >> 16);
	}
}

} /* namespace kernel */
